
#ifndef OALCHANNELGRP_H
#define OALCHANNELGRP_H

#include "soundsystem.h"

#ifdef SOUND_SYSTEM_OPENAL

#include "openalchannel.h"

//! \brief ChannelGroup
class ChannelGroup
{
public:
	ChannelGroup( const int number = 0 );
	~ChannelGroup();

	// add another OpenALChannel to this group.
	void addChannel( OpenALChannel* channel ) { mFreeChannels.push_back( channel ); }

	inline int numChannels() { return (mFreeChannels.size()+mExclusiveChannels.size()+mPlayingChannels.size()); }

	OpenALChannel* play( ALuint buffer, const bool looped = false, const bool interruptable = true, const float gain = 1.0 );
	OpenALChannel* play3D( ALuint buffer, const float x, const float y, const float z, const bool looped = false, const bool interruptable = true, const float gain = 1.0 );
	void stop();

	void setVolume( const float vol );
	inline float getVolume() const { return mVolume; }

	OpenALChannel* requestExclusiveChannel();
	void releaseExclusiveChannel( OpenALChannel* channel );

protected:
	OpenALChannel* findFreeChannel();

	int mId;
	float mVolume;

	typedef std::list<OpenALChannel*> ChannelList;
	ChannelList mFreeChannels; // This vector contains all currently unused channels for this group.
	// This vector contains all channels for exclusive use by the application. They will not be returned
	// by findFreeChannel()
	ChannelList mExclusiveChannels;
	// This vector contains all currently playing channels in this group (even when playback of a non-looped
	// sample has finished, use Channel::isPlaying() to find out whether it is actually playing).
	ChannelList mPlayingChannels;
};

#endif
#endif
