
#include "streamplayer.h"

#ifdef SOUND_SYSTEM_OPENAL


//-----------------------------------------------------------------------------
StreamPlayer::StreamPlayer()
{
#ifdef SOUND_STREAM_VORBIS
	mOggStream = new OggStream();
#endif
#ifdef SOUND_STREAM_MP3
	mMP3Stream = new MP3Stream();
#endif

	mNullStream = new SoundStream();
	mCurrentStream = mNullStream;
}

//-----------------------------------------------------------------------------
StreamPlayer::~StreamPlayer()
{
	mCurrentStream->stop();
#ifdef SOUND_STREAM_VORBIS
	delete mOggStream;
#endif
#ifdef SOUND_STREAM_MP3
	delete mMP3Stream;
#endif
	delete mNullStream;
}

//-----------------------------------------------------------------------------
enum StreamFileType {
	FT_OGG,
	FT_MP3,
	FT_UNKNOWN
};

void StreamPlayer::play(const std::string & filename)
{
	// TODO: check file type (simple look at the end of the name
	StreamFileType type;
	if ( filename.rfind(".ogg") != std::string::npos ) {
		type = FT_OGG;
	} else {
		if ( filename.rfind(".mp3") != std::string::npos ) {
			type = FT_MP3;
		} else {
			type = FT_UNKNOWN;
			// TODO: log error
			return;
		}
	}

	switch ( type ) {
	case FT_OGG:
#ifdef SOUND_STREAM_VORBIS
		mCurrentStream->stop();
		mCurrentStream = mOggStream;
		mCurrentStream->play(filename);
#endif
		break;
	case FT_MP3:
#ifdef SOUND_STREAM_VORBIS
		mCurrentStream->stop();
		mCurrentStream = mMP3Stream;
		mCurrentStream->play(filename);
#endif
		break;
	case FT_UNKNOWN:
		mCurrentStream->stop();
		mCurrentStream = mNullStream;
		mCurrentStream->play(filename);
		break;
	}
}

//-----------------------------------------------------------------------------
void StreamPlayer::stop()
{
	mCurrentStream->stop();
	mCurrentStream = mNullStream;
}

//-----------------------------------------------------------------------------
void StreamPlayer::setVolume(const float vol)
{
	mCurrentStream->setVolume(vol);
}

//-----------------------------------------------------------------------------
bool StreamPlayer::update()
{
	return mCurrentStream->update();
}

#endif

// THE END
